// qsview.cpp : implementation of the QSView class
//

#include "stdafx.h"
#include "QuickStretch.h"

#include "qsdoc.h"
#include "qsview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

QSView * g_view = 0;

/////////////////////////////////////////////////////////////////////////////
// QSView

IMPLEMENT_DYNCREATE(QSView, CScrollView)

BEGIN_MESSAGE_MAP(QSView, CScrollView)
	//{{AFX_MSG_MAP(QSView)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SCALE, OnUpdateScale)
	ON_COMMAND(ID_VIEW_SCALE, OnScale)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// QSView construction/destruction

QSView::QSView()
{
	g_view = this;
	m_scale_factor = 1.0;
}

QSView::~QSView()
{
}

BOOL QSView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// QSView drawing

void QSView::OnDraw(CDC* pDC)
{
	QSDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	ShowImage();
}

void QSView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = sizeTotal.cy = 100;
	SetScrollSizes(MM_TEXT, sizeTotal);

	m_dlg.Create(IDD_SCALE_DLG, this);
	m_dlg.ShowWindow(SW_SHOW);

	CreateClientDIB();

	CFrameWnd * frm = GetParentFrame();
	if (frm)
	{
		CRect wr, dr;
		frm->GetWindowRect(&wr);
		m_dlg.GetWindowRect(&dr);

		int left = wr.left - dr.Width() - 1;
		int top = dr.top;
		if (left < 0)
			left = wr.right + 1;
		m_dlg.SetWindowPos(&wndTop,left,top,0,0,SWP_NOSIZE|SWP_NOZORDER|SWP_SHOWWINDOW);
	}
}

void QSView::CreateClientDIB(void)
{
	CRect r;
	GetClientRect(&r);

	if ((r.Width() > 0) && (r.Height() > 0))
	{
		m_client_dib.Create(r.Width(),r.Height(),24);
		m_client_dib.PatBlt(WHITENESS);
	}
}

/////////////////////////////////////////////////////////////////////////////
// QSView diagnostics

#ifdef _DEBUG
void QSView::AssertValid() const
{
	CScrollView::AssertValid();
}

void QSView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

QSDoc* QSView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(QSDoc)));
	return (QSDoc*)m_pDocument;
}
#endif //_DEBUG

void QSView::StretchImage(double scale_factor)
{
	QSDoc * doc = GetDocument();
	if (doc)
	{
		DIBSection * dib = doc->GetDIB();

		if (dib && dib->IsCreated())
		{
			if (m_view_dib.IsCreated())
			{
				m_scale_factor = scale_factor;
				if (m_scale_factor < 0.1) m_scale_factor = 0.1;
				if (m_scale_factor > 2.0) m_scale_factor = 2.0;

				  // figure the new width and height
				unsigned long w = (int)(dib->Width() * scale_factor + 0.50);
				unsigned long h = (int)(dib->Height() * scale_factor + 0.50);
				
#ifdef PRE_STRETCH_METHOD
				if ((w != m_view_dib.Width()) || (h != m_view_dib.Height()))
				{
					m_view_dib.Close();
					
#ifdef ALTERNATE_RESIZE_METHOD
					  // StretchBlt produces good quality for expanding an
					  // image, but poor quality for shrinking. ResizeImage is an
					  // alternative to StretchBlt
					dib->ResizeImage(m_view_dib,w,h);
#else
					m_view_dib.Create(w,h,dib->GetBitCount());

					if (m_view_dib.IsCreated())
					{
						StretchBlt((HDC)*m_view_dib.GetDC(),0,0,w,h,(HDC)*dib->GetDC(),
							0,0,dib->Width(),dib->Height(),SRCCOPY);
					}
#endif
				}
#endif
				m_size.cx = w;
				m_size.cy = h;
				SetScrollSizes(MM_TEXT,m_size);

#ifdef NO_FLICKER
				if (!m_client_dib.IsCreated())
				{
					CreateClientDIB();
				}
				m_client_dib.PatBlt(WHITENESS);
#else
				  // white-out the screen before drawing image
				  // this causes flickering...to remove flickering use another
				  // DIBSection the size of the client window. Clear the client
				  // dib then draw the view dib to the client dib. Simple, but
				  // uses more memory.
				CClientDC dc(this);
				CRect r;
				GetClientRect(&r);
				dc.PatBlt(0,0,r.Width()-1,r.Height()-1,WHITENESS);
#endif

				ShowImage();
			}
		}
	}
}

void QSView::ShowImage(void)
{
	if (m_view_dib.IsCreated())
	{
		CClientDC dc(this);

		CPoint sp = GetScrollPosition();

#ifdef PRE_STRETCH_METHOD
#ifdef NO_FLICKER
		m_view_dib.Draw(m_client_dib.GetDC(),sp.x,sp.y);
		m_client_dib.Draw(&dc,0,0);
#else
		m_view_dib.Draw(&dc,sp.x,sp.y);
#endif
#else
#ifdef NO_FLICKER
		StretchBlt((HDC)*m_client_dib.GetDC(),0,0,m_size.cx,m_size.cy,(HDC)*m_view_dib.GetDC(),
			(int)(0.50 + sp.x/m_scale_factor),(int)(0.5 + sp.y/m_scale_factor),m_view_dib.Width(),m_view_dib.Height(), SRCCOPY);
		m_client_dib.Draw(&dc,0,0);
#else
		StretchBlt((HDC)dc,0,0,m_size.cx,m_size.cy,(HDC)*m_view_dib.GetDC(),
			(int)(0.50 + sp.x/m_scale_factor),(int)(0.5 + sp.y/m_scale_factor),m_view_dib.Width(),m_view_dib.Height(), SRCCOPY);
#endif
#endif
	}
}

/////////////////////////////////////////////////////////////////////////////
// QSView message handlers

void QSView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	m_view_dib.Close();
	m_dlg.ResetSliderPos();

	QSDoc * doc = GetDocument();
	if (doc)
	{
		DIBSection * dib = doc->GetDIB();
		if (dib && dib->IsCreated())
		{
			m_view_dib.Copy(*dib);
			unsigned long w = dib->Width();
			unsigned long h = dib->Height();

			m_size.cx = w;
			m_size.cy = h;
			SetScrollSizes(MM_TEXT,m_size);
			CreateClientDIB();
			m_scale_factor = 1.0;

			CPoint zero_pt(0,0);

			ScrollToPosition(zero_pt);

			ShowImage();
		}
	}
}

void QSView::OnUpdateScale(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable();
	pCmdUI->SetCheck(m_dlg.IsWindowVisible());
}

void QSView::OnScale() 
{
	m_dlg.ShowWindow(m_dlg.IsWindowVisible()?SW_HIDE:SW_SHOW);
}

void QSView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);

	if (m_hWnd)
		CreateClientDIB();
}
